package com.bcxin.runtime.apis.metas.configs;

import com.bcxin.runtime.domain.imports.components.DataImportComponentProvider;
import com.bcxin.runtime.domain.imports.repositories.DataQueueRepository;
import com.bcxin.runtime.domain.imports.services.DataQueueService;
import com.bcxin.runtime.domain.imports.services.impls.DataQueueServiceImpl;
import com.bcxin.runtime.domain.messages.repositories.MessageRepository;
import com.bcxin.runtime.domain.messages.services.MessageService;
import com.bcxin.runtime.domain.messages.services.impls.MessageServiceImpl;
import com.bcxin.runtime.domain.metas.repositories.ApplicationMetaRepository;
import com.bcxin.runtime.domain.metas.repositories.FormSyncMetaRepository;
import com.bcxin.runtime.domain.metas.repositories.FormSyncTargetMetaRepository;
import com.bcxin.runtime.domain.metas.repositories.TaskMetaRepository;
import com.bcxin.runtime.domain.metas.services.ApplicationMetaService;
import com.bcxin.runtime.domain.metas.services.FormSyncMetaService;
import com.bcxin.runtime.domain.metas.services.TaskMetaService;
import com.bcxin.runtime.domain.metas.services.impls.ApplicationMetaServiceImpl;
import com.bcxin.runtime.domain.metas.services.impls.FormSyncMetaServiceImpl;
import com.bcxin.runtime.domain.metas.services.impls.TaskMetaServiceImpl;
import com.bcxin.runtime.domain.repositories.syncs.repositories.JpaDataMapSyncRepository;
import com.bcxin.runtime.domain.syncs.components.DbSyncExecutor;
import com.bcxin.runtime.domain.syncs.repositories.DataSyncMapRepository;
import com.bcxin.runtime.domain.syncs.repositories.DataSyncLogRepository;
import com.bcxin.runtime.domain.syncs.repositories.FileSyncQueueRepository;
import com.bcxin.runtime.domain.syncs.services.DataSyncMapService;
import com.bcxin.runtime.domain.syncs.services.DataSyncService;
import com.bcxin.runtime.domain.syncs.services.impls.DataSyncMapServiceImpl;
import com.bcxin.runtime.domain.syncs.services.impls.DataSyncServiceImpl;
import com.bcxin.saas.core.AppConfigProperty;
import com.bcxin.saas.core.components.HttpRequestProvider;
import com.bcxin.saas.core.components.JsonProvider;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;

@Qualifier("com.bcxin.runtime.apis.metas.configs.SetupConfig")
@Configuration("com.bcxin.runtime.apis.metas.configs.SetupConfig")
@ComponentScan(basePackages = {"com.bcxin.runtime.domain.repositories.metas",
                               "com.bcxin.runtime.domain.repositories.imports",
                               "com.bcxin.runtime.domain.repositories.messages"})
public class SetupConfig {
    public SetupConfig() {

    }


    @Bean("com.bcxin.runtime.apis.metas.configs.SetupConfig.appConfigProperty")
    public AppConfigProperty appConfigProperty(
            @Value("${myapps.storage.root}") String rootPath,
            @Value("${myapps.domain.url:}") String domainUrl,
            @Value("${myapps.domain.needTDocument:false}")boolean needTDocument) {
        return AppConfigProperty.create(rootPath, domainUrl,needTDocument);
    }

    @Bean(name = "com.bcxin.runtime.apis.metas.configs.SetupConfig.applicationMetaService")
    public ApplicationMetaService applicationMetaService(ApplicationMetaRepository applicationMetaRepository) {
        return new ApplicationMetaServiceImpl(applicationMetaRepository);
    }

    @Bean(name = "com.bcxin.runtime.apis.metas.configs.SetupConfig.formSyncMetaService")
    public FormSyncMetaService formSyncMetaService(ApplicationMetaRepository applicationMetaRepository,
                                                   FormSyncMetaRepository formSyncMetaRepository,
                                                   FormSyncTargetMetaRepository formSyncTargetMetaRepository,
                                                   JsonProvider jsonProvider) {
        return new FormSyncMetaServiceImpl(formSyncMetaRepository, applicationMetaRepository, jsonProvider, formSyncTargetMetaRepository);
    }

    @Bean(name = "com.bcxin.runtime.apis.metas.configs.SetupConfig.taskMetaService")
    public TaskMetaService taskMetaService(TaskMetaRepository taskMetaRepository,
                                           JsonProvider jsonProvider,
                                           ApplicationMetaRepository applicationMetaRepository) {
        return new TaskMetaServiceImpl(taskMetaRepository, jsonProvider, applicationMetaRepository);
    }

    @Bean(name = "com.bcxin.runtime.apis.metas.configs.SetupConfig.dataQueueService")
    public DataQueueService dataQueueService(DataQueueRepository dataQueueRepository,
                                             JsonProvider jsonProvider,
                                             DataImportComponentProvider dataImportComponentProvider) {
        return new DataQueueServiceImpl(dataQueueRepository, jsonProvider, dataImportComponentProvider);
    }

    @Bean(name = "com.bcxin.runtime.apis.metas.configs.SetupConfig.dataMapSyncService")
    public DataSyncService dataMapSyncService(DataSyncMapRepository dataMapSyncRepository,
                                              DataSyncLogRepository dataSyncLogRepository,
                                              JsonProvider jsonProvider,
                                              DbSyncExecutor dbSyncExecutor,
                                              HttpRequestProvider httpRequestProvider,
                                              AppConfigProperty appConfigProperty,
                                              FileSyncQueueRepository fileSyncQueueRepository) {
        return new DataSyncServiceImpl(dataMapSyncRepository, dataSyncLogRepository,
                jsonProvider, dbSyncExecutor, httpRequestProvider, appConfigProperty, fileSyncQueueRepository);
    }

    @Bean(name = "com.bcxin.runtime.apis.metas.configs.SetupConfig.dataSyncMapService")
    public DataSyncMapService dataSyncMapService(JpaDataMapSyncRepository jpaDataMapSyncRepository,
                                                 JsonProvider jsonProvider) {
        return new DataSyncMapServiceImpl(jpaDataMapSyncRepository, jsonProvider);
    }

    @Bean(name = "com.bcxin.runtime.apis.metas.configs.SetupConfig.httpRequestProvider")
    public HttpRequestProvider httpRequestProvider(JsonProvider jsonProvider) {
        return new HttpRequestProvider.HttpRequestProviderImpl(jsonProvider);
    }

    @Bean(name = "com.bcxin.runtime.apis.metas.configs.SetupConfig.messageService")
    public MessageService messageService(MessageRepository messageRepository,JsonProvider jsonProvider) {
        return new MessageServiceImpl(messageRepository, jsonProvider);
    }
}
